using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

using Microsoft.Modeling;

namespace LightSwitch
{
    /// <summary>
    /// An example model program.
    /// </summary>
    static class LightSwitchModelProgram
    {
        static bool switchState = false; // Where True is on and False is off

        /// <summary>
        /// The AcceptingStateCondition dictates what state is valid for
        ///   the model to finish on.  This insures that when the model
        ///   is run again that it is in a clean state.  In this case
        ///   we have to make sure that the System Under Test(SUT) is reset
        ///   to the off state before closing, otherwise if you run 
        ///   On-the-fly tests, the model will sometimes fail because 
        ///   the last test left the SUT in an invalid state.
        /// </summary>
        /// <returns>True if the switch is off, False if it is on.</returns>
        [AcceptingStateCondition]
        static bool IsInAcceptingState()
        {
            return switchState == false;
        }

        /// <summary>
        /// Maps to the "FlipSwitch()" method, but will
        /// only fire when the switch is supposed to be Off.
        /// So it always turns it on.
        /// </summary>
        [Rule(Action = "FlipSwitch()/result")]
        static bool TurnOn()
        {
            Condition.IsTrue(!switchState);
            switchState = true;
            return switchState;
        }

        /// <summary>
        /// Maps to the "FlipSwitch()" method, but will
        /// only fire when the switch is supposed to be Off.
        /// So it always turns it on.
        /// </summary>
        [Rule(Action = "FlipSwitch()/result")]
        static bool TurnOff()
        {
            Condition.IsTrue(switchState);
            switchState = false;
            return switchState;
        }
    }
}
